PTTDB = PTTDB or {scale = 1}

local f

local bars = {}
local barptrs = {}
local band = bit.band
local tinsert = tinsert
local select = select
local floor = math.floor
local ipairs = ipairs
local UnitExists = UnitExists
local UnitName = UnitName
local UnitGUID = UnitGUID
local UnitIsUnit = UnitIsUnit

local classcolors = RAID_CLASS_COLORS
local testing
local ref

do
	local function rgbhex(r, g, b)
		return format("|cff%02x%02x%02x", r * 255, g * 255, b * 255)
	end
	
	for k,v in pairs(RAID_CLASS_COLORS) do
		v.hex = rgbhex(v.r, v.g, v.b)
	end
end

local pvpicons = {
	["Alliance"] = "Interface\\Icons\\INV_Jewelry_Necklace_37",
	["Horde"] = "Interface\\Icons\\INV_Jewelry_Necklace_38",
	["Human"] = "Interface\\Icons\\Spell_Shadow_Charm",
}


local trinketcd = 120
local barwidth = 242
local barheight = 20
local iconsize = 20
local timeformat = "%d:%02d" 
local bartexture = "Interface\\AddOns\\PvPTrinketTracker\\Images\\barSmooth"


	
local inarena = function() return (select(2,IsInInstance()) == "arena") end

local function PTT_CreateBars(self)
	for i=1,5 do
		local bar = CreateFrame("StatusBar","PTT_Bar"..i,self)
		bar.unit = "arena"..i
		local unit = bar.unit
		bar:Hide()
	
	
		local _shower = CreateFrame("Button",nil,UIParent,"SecureUnitButtonTemplate")
		bar._shower = _shower
		_shower.unit = bar.unit
		SecureUnitButton_OnLoad(_shower,_shower.unit)
		RegisterUnitWatch(_shower)
		
		_shower:EnableMouse(true)
		_shower:SetWidth(barwidth)
		_shower:SetHeight(barheight)
		
		_shower:SetScript("OnShow",function(self)	
			if select(2,UnitRace(bar.unit)) == "Human" then
				bar.icon:SetTexture(pvpicons["Human"])
			else
				bar.icon:SetTexture(pvpicons[UnitFactionGroup(bar.unit)])
			end
			bar:Show()
		end)
		
		_shower:SetPoint("TOP",ref,"BOTTOM",0,0)
		

		bar:SetWidth(barwidth)
		bar:SetHeight(barheight)
		bar:SetStatusBarTexture(bartexture)
		
		bar:SetPoint("TOP",ref,"BOTTOM",0,0)
		
		ref = _shower
		
		bar:SetMinMaxValues(0,1)

		local bg = bar:CreateTexture(nil,"BACKGROUND")
		bg:SetAllPoints(true)
		bg:SetTexture(bartexture)
		bar.bg = bg
		
		local highlight = bar:CreateTexture(nil,"ARTWORK")
		highlight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
		highlight:SetAllPoints(true)
		highlight:SetBlendMode("ADD")
		highlight:Hide()
		bar.highlight = highlight
		
		local spark = bar:CreateTexture(nil, "BORDER");
		spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark");
		spark:SetBlendMode("ADD")
		spark:SetHeight(barheight * 2.25)
		spark:Show()
		bar.spark = spark
		
		local icon = bar:CreateTexture(nil,"ARTWORK")
		icon:SetWidth(iconsize)
		icon:SetHeight(iconsize)
		icon:SetPoint("RIGHT",bar,"LEFT")
		icon:SetTexCoord(0.07,0.9,0.07,0.90)
		bar.icon = icon
		
		local name = bar:CreateFontString(nil,"ARTWORK","GameFontNormal")
		name:SetJustifyH("LEFT")
		name:SetPoint("LEFT",bar,5,-1)
		bar.name = name
		
		local timetext = bar:CreateFontString(nil,"ARTWORK","ChatFontNormal")
		timetext:SetJustifyH("RIGHT")
		timetext:SetPoint("RIGHT",bar)
		bar.timetext = timetext
		
		bar.SetColor = function(r,g,b,a) 
			bar:SetStatusBarColor(r,g,b,a)
			bar.bg:SetVertexColor(r,g,b,a/4)
		end
		
		bar.Start = function(testtime)
			bar:Show()
			bar.SetColor(0,1,0,.4)
			if testtime then
				bar.starttime = testtime
			else
				bar.starttime = GetTime()	
			end
			bar.active = true
			bar.spark:Show()
		end
		
		bar.Stop = function()
			if bar.hideOnEnd then
				bar:Hide()
				bar.hideOnEnd = nil
			end
			bar.SetColor(1,0,0,.4)
			bar:SetValue(1)
			bar.icon:SetAlpha(1)
			
			bar.active = false
			bar.timetext:SetText("")
			bar.spark:Hide()
		end
		
		bar.UpdateColor = function()
			if UnitIsDead(unit) then
				bar.isDead = true
				bar.SetColor(0.5,0.5,0.5,0.6)
			elseif bar.isDead then -- Unit rezzed
				bar.isDead = nil
				if bar.active then
					bar.SetColor(0,1,0,.4)
				else
					bar.SetColor(1,0,0,.4)
				end
			end
		end
		
		bar.UpdateName = function ()
			local name = UnitName(unit)
			local _,class = UnitClass(unit)
			local color = classcolors[class].hex
			bar.name:SetText(color..name.."|r")
		end
		
		bar.Stop()
		tinsert(bars,bar)	
	end
end

local function PTT_UpdateBar(bar)	
	barptrs[UnitGUID(bar.unit)] = bar
	bar.UpdateName()
	bar.UpdateColor()
end


local function PTT_UpdateHighlights()
	for _,bar in ipairs(bars) do
		if UnitExists("target") and UnitGUID("target") == UnitGUID(bar.unit) then
			bar.highlight:Show()
		else
			bar.highlight:Hide()
		end
	end
end

local time = 0
local function PTT_OnUpdate(self,elapsed)
	time = time + elapsed
	if time > 0.04 then
		for _,bar in ipairs(bars) do
			if bar.active then
				bar.timeleft = bar.starttime + trinketcd - GetTime()
				if bar.timeleft > 0 then 
					bar:SetValue(bar.timeleft / trinketcd)
					bar.timetext:SetFormattedText(timeformat, floor(bar.timeleft/60), bar.timeleft % 60)
					bar.spark:ClearAllPoints()
					bar.spark:SetPoint("CENTER",bar,"LEFT",(barwidth*bar.timeleft/trinketcd)+0.5,0)
					if bar.timeleft <= 15 then
						local alpha = bar.timeleft % 2
						if alpha > 1 then
							bar.icon:SetAlpha(2 - alpha)
						else
							bar.icon:SetAlpha(alpha)
						end
					else
						bar.icon:SetAlpha(1)
					end
				else
					bar.Stop()
				end
			end
			if not testing and bar:IsVisible() and UnitExists(bar.unit) then -- can't check not UnitExist because of stealth/shadowmeld/invis
				PTT_UpdateBar(bar)
			end
		end
		time = 0
	end
end


local names = {"Riannese","Amygnaw","Teklivklo","Wgarnigrednmars","Lexec"}
local testclasses = {"WARRIOR","DRUID","SHAMAN","PRIEST","MAGE"}
   
local function PTT_Test()
	if inarena() then return end
	testing = true
	for k,bar in ipairs(bars) do
		local rand = math.ceil(random() * 3)
		if rand == 1 then
			bar.icon:SetTexture(pvpicons["Human"])
		elseif rand == 2 then
			bar.icon:SetTexture(pvpicons["Alliance"])
		else
			bar.icon:SetTexture(pvpicons["Horde"])
		end
		bar.name:SetText(classcolors[testclasses[k]].hex..names[k].."|r")
		bar.hideOnEnd = true
		bar.Start(GetTime() - (random() * trinketcd))
	end
end

local function PTT_SavePosition()
	local point, _, relativePoint, xOfs, yOfs = f.button:GetPoint()
	if not PTTDB.Position then 
		PTTDB.Position = {}
	end
	PTTDB.Position.point = point
	PTTDB.Position.relativePoint = relativePoint
	PTTDB.Position.xOfs = xOfs
	PTTDB.Position.yOfs = yOfs
end

local function PTT_LoadPosition()
	f.button:ClearAllPoints()
	if PTTDB.Position then
		f.button:SetPoint(PTTDB.Position.point,UIParent,PTTDB.Position.relativePoint,PTTDB.Position.xOfs,PTTDB.Position.yOfs)
	else
		f.button:SetPoint("CENTER", UIParent, "CENTER")
	end
end

local function PTT_ApplyOptions()
	if PTTDB.lock then f.button:Hide() else
	f.button:Show() end f:SetScale(PTTDB.scale)
	for k,v in ipairs(bars) do v._shower:SetScale(PTTDB.scale) end
end

local function PTT_OnLoad(self)
	local button = CreateFrame("Button","PTT_Button",UIParent)
	button:SetMovable(true)
	button:EnableMouseWheel(1)
	button:SetWidth(130)
	button:SetHeight(16)
	local font = CreateFont("PTTFont")
	font:SetFont(GameFontNormal:GetFont(),12)
	font:SetJustifyH("CENTER")
	button:SetNormalFontObject(font)
	button:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	button:SetText("PvP Trinket Tracker")
	
	local function PTT_GameTooltipUpdate()
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:AddLine("PvP Trinket Tracker Anchor")
		GameTooltip:AddLine("Left-Click to move anchor")
		GameTooltip:AddLine("Right-Click to test bars")
		GameTooltip:AddLine('Type "/ptt lock" to toggle anchor')
		GameTooltip:AddLine('Scroll up to increase scale')
		GameTooltip:AddLine('Scroll down to decrease scale')
		GameTooltip:AddLine('Scale: ' .. (PTTDB.scale or 1))
		GameTooltip:Show()
	end
	
	button:SetScript("OnEnter",function(self)
		PTT_GameTooltipUpdate()
	end)
	button:SetScript("OnLeave",function(self)
		GameTooltip:Hide()
	end)
	button:SetScript("OnMouseDown", 
		function(self, button)
			if button == "LeftButton" then
				self:StartMoving()
			end 
	end)
	button:SetScript("OnMouseUp", 
		function(self, button)
			if button == "LeftButton" then
				self:StopMovingOrSizing()
				PTT_SavePosition()
			elseif button == "RightButton" then
				PTT_Test()
			end
	end)
	
	button:SetScript("OnMouseWheel",function(self,a1)
		if a1 == 1 then
			PTTDB.scale = (PTTDB.scale or 1) + 0.05
		else
			local newscale = (PTTDB.scale or 1) - 0.05
			if newscale <= 0 then newscale = 0.1 end
			PTTDB.scale = newscale
		end
		PTT_GameTooltipUpdate()
		PTT_ApplyOptions()
	end)
	
	self.button = button
	
	ref = button
	
	
	PTT_LoadPosition()
	SlashCmdList["PTT"] = function(msg)
		msg = string.lower(msg)
		if msg == "lock" then
			PTTDB.lock = not PTTDB.lock
		elseif msg == "test" then
			PTT_Test()
		end
		PTT_ApplyOptions()
	end
 	SLASH_PTT1 = "/ptt"
 	
	PTT_CreateBars(self)
	PTT_ApplyOptions()
	self:SetScript("OnUpdate",PTT_OnUpdate)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	ChatFrame1:AddMessage("PvP Trinket Tracker by Kollektiv. Type /ptt lock to show/hide anchor")
end

local function PTT_COMBAT_LOG_EVENT_UNFILTERED(_, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName)
	if not inarena() then return end
	if barptrs[srcGUID] and eventtype == "SPELL_CAST_SUCCESS" and band(srcFlags,0x00000040) == 0x00000040 and (spellName == "PvP Trinket" or spellName == "Every Man for Himself") then  
		barptrs[srcGUID].Start()
	end
end

f = CreateFrame("Frame","PTT_Frame",UIParent)
f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:SetScript("OnEvent",function(self,event,...)
	if event == "VARIABLES_LOADED" then
		PTT_OnLoad(self)
	elseif event == "PLAYER_ENTERING_WORLD" then
		for _,v in ipairs(bars) do
			v:Hide()
			v.Stop()
			testing = false
		end
		wipe(barptrs)
	elseif event == "PLAYER_TARGET_CHANGED" then
		PTT_UpdateHighlights()
	else
		PTT_COMBAT_LOG_EVENT_UNFILTERED(...)
	end
end)
